
@rem FASMLIB build script
@rem you need to have these in PATH:
@rem   fasm.exe
@rem   ml.exe  		(MASM)
@rem   nasm.exe
@rem   link.exe
@rem   ld.exe		(win32 version of GNU linker, to produce ELF executable)
@rem   def2lib.exe	(Vortex Def2Lib utility, to build import library)
@rem   sed.exe		(win32 version of linux stream editor utility)
@rem   asmdoc.bat	(batch file to call AsmDoc utility)
@rem   xmllint.exe	(XML preparser, part of libxml2)
@rem   saxon.bat	(batch file to call Saxon 6 XSLT processor)
@rem   hhc.exe		(HTML Help Compiler, part of HTML Help Workshop)
@rem You also must have XSL DocBook installed, and set path in "doc\style.xsl"

@prompt $G$S

@SET LOG="%~dp1build-%1.log"

@if "%1" == "" goto usage
@if "%1" == "dev" goto end
@SET DEST=fasmlib-%1


@if not exist "%DEST%" goto copy

@echo Deleting directory "%DEST%"
@rd /S /Q %DEST%

:copy
@echo Copying "dev" to "%DEST%"
@mkdir %DEST%
@xcopy /Q /E dev\bin "%DEST%\bin\" >%LOG% 2>&1
@xcopy /Q /E dev\src "%DEST%\src\" >%LOG% 2>&1
@xcopy /Q /E dev\include "%DEST%\include\" >%LOG% 2>&1
@xcopy /Q /E dev\examples "%DEST%\examples\" >%LOG% 2>&1
@xcopy /Q /E dev\doc "%DEST%\doc\" >%LOG% 2>&1
@xcopy /Q /E dev\test "%DEST%\test\" >%LOG% 2>&1
@copy dev\whatsnew.txt "%DEST%\whatsnew.txt" >%LOG% 2>&1
@cd "%DEST%"
@set FASMLIB=%CD%

@echo Building import headers
@copy include\masm\procs.inc test\masm-include-procs.inc >%LOG% 2>&1
@cd include
@call make.bat >>%LOG% 2>&1
@del make.bat
@cd ..

@cd bin

@echo Building binary versions
@echo - DLL
@cd dll
@call make.bat >>%LOG% 2>&1
@cd ..
@rd /S /Q dll\

@echo - ELF
@cd elf
@call make.bat >>%LOG% 2>&1
@cd ..
@rd /S /Q elf\

@echo - MS-COFF
@cd ms-coff
@call make.bat >>%LOG% 2>&1
@cd ..
@rd /S /Q ms-coff\

@echo - MS-COFF2
@cd ms-coff2
@call make.bat >>%LOG% 2>&1
@cd ..
@rd /S /Q ms-coff2\

@echo - LIB
@cd lib
@call make.bat >>%LOG% 2>&1
@cd ..
@rd /S /Q lib\

@echo - LIB2
@cd lib2
@call make.bat >>%LOG% 2>&1
@cd ..
@rd /S /Q lib2\

@cd ..

@echo Running tests
@cd test
@call test.bat >>%LOG% 2>&1
@cd ..
@rd /S /Q test

@echo Building examples
@cd examples
@call make.bat >>%LOG% 2>&1
@rd /S /Q _lib
@del make.bat
@cd ..

@echo Building documentation
@cd doc
@echo - generating documentation from AsmDoc comments >con
@call build_xml.bat >>%LOG% 2>&1
@echo - building CHM version >con
@call make-chm.bat >>%LOG% 2>&1
@echo - building HTML version >con
@call make-html.bat >>%LOG% 2>&1
@rd /S /Q doc
@rd /S /Q docbook
@rd /S /Q gfx
@del /Q *.bat
@del /Q chm.css
@del /Q chm-style.xsl
@del /Q html-style.xsl
@cd ..

@echo Deleting extra files
@del bin\public.inc
@del bin\internal.inc
@del doc\build.txt
@rd /S /Q doc\proc
@del src\conv\_qfp_mul.inc
@del src\conv\_qfp_div.inc
@del src\conv\fp_t.inc

@echo Done
@pause

@goto end

:usage
@echo uasge: build.bat X.Y.Z
@echo    where X.Y.Z is version number
@pause
@goto end

:end
@cd C:\dev\fasmlib
