;helper file to build FASMLIB in .LIB format
;for use with MASM (names decorated in MASM style)

;do not compile this file directly,
;run make-win32.bat to build bin/lib2.lib file

format MS COFF

;FASMLIB settings
SYSTEM			equ win32      ;destination platform ("win32" or "linux")
INCLUDE_ALL_ERRORS	equ 1	       ;include unused errors too
INCLUDE_ALL_PROCS	equ 1	       ;include unused procedures too
INCLUDE_ALL_DATA	equ 1	       ;include unused data too
INCLUDE_TIMEDATE	equ 0	       ;include string hodling time/date of compilation of FASMLIB
;HEAP_INIT_SIZE          equ 65536      ;default heap size, must be power of two (only valid when FASMLIB heap manager is used)
;HEAP_SIZE_ALIGN         equ 4096       ;aligning heap size, must be power of two (only valid when FASMLIB heap manager is used)
;HEAP_BLOCK_ALIGN        equ 64         ;aligning heap block size, must be power of two (only valid when FASMLIB heap manager is used)
DISPLAY_FASMLIB_INFO	equ 0	       ;display overall FASMLIB info during compilation
;DISPLAY_PROC_INFO       equ 0          ;display info about procedures during compilation, addresses must be fixed
DISPLAY_MODULE_INFO	equ 0	       ;display info about FASMLIB modules compilation
DISPLAY_STRUC_INFO	equ 0	       ;display info about FASMLIB structures compilation


;FASMLIB headers
virtual at 0
  include '../../src/fasmlib/data.inc'
  include '../../src/fasmlib/proc.inc'
  include '../../src/fasmlib/ifndef.inc'

  ;define error codes only
  macro errtable [name, text] {
    common
      local code
      code = 0
    forward
      name = code
      code = code+1
    common
      err.count = code
  }
  include '../../src/err/table.inc'

  include '../../src/fasmlib/jcc.inc'
  include '../../src/stream/_interface.inc'
  include '../../src/fstream/_interface.inc'
  include '../../src/mstream/_interface.inc'
end virtual


;code section
section '.text' code readable executable align 16


;include target file (must be set with environment variable)
match f, FILE {
  include f
}


;data section
section '.data' data readable writeable align 16
IncludeIData
IncludeUData


;public FASMLIB symbols
DOT fix '_'
macro fasmlib_proc name*, strname*, argsize*
{
  ifndef name
    if used name
      extrn '_'#strname#'@'#`argsize as name
    end if
  else
    public name as '_'#strname#'@'#`argsize
  end if
}
macro fasmlib_data name*, strname*, type
{
  ifndef name
    if used name
      if type eq
	extrn '_'#strname as name
      else
	extrn '_'#strname as name:type
      end if
    end if
  else
    public name as '_'#strname
  end if
}
include '../public.inc'
include '../internal.inc'


;win32 imports that fasmlib uses
macro import name, size {
  if used name
    extrn '__imp__'#`name#'@'#`size as name:dword
  end if
}

import ExitProcess, 4
import CreateFileA, 28
import GetLastError, 0
import CloseHandle, 4
import SetFilePointer, 16
import ReadFile, 20
import WriteFile, 20
import GetProcessHeap, 0
import HeapAlloc, 12
import HeapFree, 12
import HeapReAlloc, 16
import VirtualAlloc, 16
import VirtualFree, 12
import GetStdHandle, 4
