;internal procedure to be used with heap manager's mem.init.
;Don't call this procedure directly.

;; @name mem.realloc_heap
;; @desc
;;    This is internal procedure of heap manager, that handles
;;    platform-dependent aspect of resizing entire heap.
;;    Do not call this procedure.
;; @arg heapptr
;;    Pointer to heap block returned by {mem.alloc_heap} or by this procedure.
;; @arg oldsize
;;    Current size of heap. Must be provided by caller.
;; @arg newsize
;;    Desired size of heap.
;; @ret
;;    CF set on error, otherwise
;;    EAX = heapptr
;; @err ERR_INVALID_POINTER
;;    Heap pointer is invalid
;; @err ERR_OUT_OF_MEM
;;     Not enough memory
;; @err ERR_ZERO_SIZE
;;     newsize = 0
;; @err ERR_OUT_OF_RANGE
;;     size is negative (or larger than 7FFFFFFFh if viewed as unsigned)
;; @err ERR_UNKNOWN
;;     System returned error that wasn't translated to FASMLIB error.
;;     You can {contact} author about this, and translation can be added.
;; @warn Don't call this procedure!
;; @warn Heap cannot be moved during resizing.
;; @note Data in heap must be preserved.
;; @note If block is enlarging, then contents of added memory are unknown
proc mem.realloc_heap heapptr, newsize, oldsize
	push	ebx

	;call sys_brk(blockptr)
	mov	ebx, [heapptr] ;pointer to block
	add	ebx, [newsize]
	mov	eax, 45 	;sys_brk
	int	80h

	;check error
	cmp	eax, -4096
	ja	.error_brk

.rnc:	clc
.r:	pop	ebx
	ret
.rc:	stc
	jmp	.r

.error_brk:
	;catch invalid block pointer
	cmp	eax, -22 ;EINVAL
	jne	@f
	mov	eax, ERR_INVALID_POINTER
	jmp	.rc
@@:

	;catch out-of-memory
	cmp	eax, -12 ;ENOMEM
	jne	@f
	mov	eax, ERR_OUT_OF_MEM
	jmp	.rc
@@:

	mov	eax, ERR_UNKNOWN
	jmp	.rc

.error_newsize_zero:
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc

.error_newsize_too_big:
	mov	eax, ERR_OUT_OF_RANGE
	jmp	.rc
endp