;; @name fileio.tell
;; @desc
;;    Returns current position in file. Position is 64bit value.
;; @arg handle
;;    File handle (returned by {fileio.open}, {fileio.create} or {fileio.append}).
;; @ret
;;    CF set on error, otherwise
;;    EDX:EAX = current position in file
;; @err ERR_INVALID_HANDLE
;;    Invalid file handle.
;; @err ERR_UNKNOWN
;; @note This is platform-dependent procedure
proc fileio.tell, handle
local high:DWORD
	push	ecx

	;move 0 bytes from current position
	mov	[high], 0
	push	1		;current position
	lea	eax, [high]
	push	eax			;ptr to high order dword of distance to move
	push	0			;distance to move = 0
	push	[handle]
	call	[SetFilePointer]
	cmp	eax,-1
	je	.error_SetFilePointer

	;save return value in ECX
	mov	ecx, eax

	;check for error
	;we have to call GetLastError when -1 is returned
	cmp	eax, -1
	jne	@f
	call	[GetLastError]
	cmp	eax, 0
	jne	.error_SetFilePointer
	@@:

	;return new position
	mov	edx, [high]
	mov	eax, ecx
	clc

.r:	pop	ecx
	ret

.rc:	stc
	jmp	.r

.error_SetFilePointer:
	call	[GetLastError]

	;handle invalid type of handle :)
	push	ERR_INVALID_HANDLE
	cmp	eax, 6		;ERROR_INVALID_HANDLE
	je	@f
	cmp	eax, 132	;ERROR_SEEK_ON_DEVICE
	je	@f
	mov	[esp],eax
@@:	pop	eax
	je	.rc

	mov	eax, ERR_UNKNOWN
	jmp	.rc
endp