idata {
  mem.initialized dd 0		;0 = not initialized, 1=initialized
}

udata {
  mem.heap	  dd ?		       ;windows standard heap handle
  mem.alloc_count dd ?
}

;===================================================================================
;; @name mem.init
;; @desc
;;     Initializes FASMLIB memory managent.
;;     You need to call this procedure to use any of following procedures:
;;     {mem.alloc}, {mem.alloc0}, {mem.realloc}, {mem.realloc0}, {mem.free}, {mem.size}.
;; @ret
;;     CF set on error
;; @err ERR_OUT_OF_MEM
;;     Not enough memory
;; @err ERR_UNKNOWN
;;     System returned error that wasn't translated to FASMLIB error.
;;     You can {contact} author about this, and translation can be added.
;; @dep mem.alloc_heap
;;     Only on platforms where FASMLIB heap manager is used.
;; @warn Don't forget to always uninitialize memory management with {mem.uninit}!
;; @note This procedure is called by {fasmlib.init}.
;;       You can use it for lazy initialization of entire library.
;; @note If memory management is already initialized, this procedure does nothing
;; @note This is platform-specific procedure
proc mem.init
	push	ecx edx

	cmp	[mem.initialized],0
	jne	.rnc

	;get heap pointer
	call	[GetProcessHeap]

	or	eax, eax
	jz	.error_GetProcessHeap

	mov	[mem.heap], eax
	mov	[mem.initialized], 1
	mov	[mem.alloc_count], 0

.rnc:	clc
.r:	pop	edx ecx
	ret
.rc:	stc
	jmp	.r

.error_GetProcessHeap:
	mov eax, ERR_UNKNOWN
	jmp .rc
endp