;; @name stdstream.uninit
;; @desc
;;    Uninitializes standard streams: {stdin}, {stdout}, {stderr}.
;;    You should call this procedure after you are done using these streams.
;; @ret
;;    CF set on error
;; @err ERR_MODULE_NOT_INITIALIZED
;;    Module wasn't initialized. Initialize module with {stdstream.init} first.
;; @warn
;;    If you don't call this procedure, you may recieve ERR_MEMORY_LEAK
;;    from {mem.uninit}, because stream buffers will still be allocated.
;; @note
;;    This procedure is called by {fasmlib.uninit}.
;;    You can use it for lazy uninitialization of entire library.
;; @note
;;    {stderr} is not uninitialized, so it can be always used to print error.
;;    It also doesn't need to be initialized, because it is static data, and
;;    without buffering
proc stdstream.uninit

	;check if module is initialized
	cmp	[stdstream.initialized], 0
	je	.error_not_initialized

	mov	[stdstream.initialized], 0

	;stdin
if used stdin | INCLUDE_ALL_DATA eq 1
	libcall stream.close, stdin
	jc	.rc
end if

.rnc:	clc
.r:	ret
.rc:	stc
	jmp	.r


.error_not_initialized:
	mov	eax, ERR_MODULE_NOT_INITIALIZED
	jmp	.rc
endp
