;=============================================================================
; extconv.hex2id_size
; desc: INTERNAL! ignore this :]
;       performs job for hex_ib, hex_iw, hex_id procedures
;=============================================================================
proc extconv.hex2id_size string, strlen, size, allowplus, len
local sign:DWORD
	push	esi ecx ebx

	;error is strlen=0
	cmp	[strlen], 0
	je	.error_zero_size

	;ESI = string
	mov	esi, [string]

	;ECX = count of converted chars
	mov	ecx, 0

	;get sign
	mov	[sign], 0
	cmp	byte [esi], '-'
	jne	taken @f
	inc	esi
	inc	ecx
	mov	[sign], 1
	jmp	.sign_done
	@@:
	cmp	[allowplus], 0
	je	.sign_done
	cmp	byte [esi], '+'
	jne	.sign_done
	inc	esi
	inc	ecx
.sign_done:

	;check first digit
	cmp	byte [esi], '0'
	jb	.error_not_hex
	cmp	byte [esi], '9'
	jbe	@f
	cmp	byte [esi], 'A'
	jb	.error_not_hex
	cmp	byte [esi], 'F'
	jbe	@f
	cmp	byte [esi], 'a'
	jb	.error_not_hex
	cmp	byte [esi], 'f'
	ja	.error_not_hex
	@@:

	;EBX = final value, for now 0
	mov	ebx, 0

	;loop
.digit_loop:
	;AL=char
	mov	al, [esi]
	inc	esi

	;end if it isn't hex digit
	cmp	al, '0'
	jb	.done
	cmp	al, '9'
	jbe	.numeric
	cmp	al, 'A'
	jb	.done
	cmp	al, 'F'
	jbe	.upcase_letter
	cmp	al, 'a'
	jb	.done
	cmp	al, 'f'
	ja	.done

.lowcase_letter:
	sub	al, 'a'-10
	jmp	.add_digit

.upcase_letter:
	sub	al, 'A'-10
	jmp	.add_digit

.numeric:
	sub	al, '0'

.add_digit:
	;add digit
	shl	ebx,4
	movzx	eax, al
	add	ebx, eax

	;check overflow
	cmp	ebx, [size]
	ja	.overflow

	;continue loop
	inc	ecx
	cmp	ecx, [strlen]
	jb	.digit_loop
.done:

	;fill len
	mov	eax, [len]
	cmp	eax, 0
	je	@f
	mov	[eax], ecx
	@@:

	;set sign and check overflow
	cmp	[sign], 0
	jne	.negative
.positive:
	cmp	ebx, [size]
	jae	.overflow
	jmp	.sign_set
.negative:
	cmp	ebx, [size]
	ja	.overflow
	neg	ebx
.sign_set:

	;return EBX
	mov	eax, ebx

	clc
.r:	pop	ebx ecx esi
	ret
.rc:	stc
	jmp	.r


.overflow:
	;set len
	mov	eax, [len]
	cmp	eax, 0
	je	@f
	mov	dword [eax], ecx
	@@:

	;return error
	mov	eax, ERR_OUT_OF_RANGE
	jmp	.rc

.error_zero_size:
	;set len to 0
	mov	eax, [len]
	cmp	eax, 0
	je	@f
	mov	dword [eax], 0
	@@:

	;return error
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc


.error_not_hex:
	;set len to 0
	mov	eax, [len]
	cmp	eax, 0
	je	@f
	mov	dword [eax], 0
	@@:

	;return error
	mov	eax, ERR_NOT_HEXADECIMAL
	jmp	.rc
endp