;; @name conv.dec_d
;; @desc
;;    Converts decimal string to unsigned dword (32bit) value.
;; @arg string
;;    Pointer to string to convert.
;; @arg buflen
;;    Size of string's buffer.
;;    You can set this to -1 (FFFFFFFFh) if string is quaranteed to be properly zero-terminated.
;; @ret
;;    CF=1 on error, otherwise
;;    EAX = value.
;; @err ERR_NOT_DECIMAL
;;    First character of string is not decimal digit.
;; @err ERR_OUT_OF_RANGE
;;    Value is > 4294967295.
;; @err ERR_ZERO_SIZE
;;    strlen=0
;; @note This procedure converts digits until it reaches non-digit character,
;;       or until value overflows.
;; @note To get number of converted characters, use {extconv.dec_d}.
;; @nodeps
proc conv.dec_d string, buflen
	libcall extconv.dec2d_size, [string], [buflen], 4294967295, 0
	ret
endp