;; @name extconv.dec_id
;; @desc
;;    Converts decimal string to signed dword (32bit) value.
;;    This is extended version, for simpler version see {conv.dec_id}.
;; @arg string
;;    Pointer to string to convert.
;; @arg buflen
;;    Size of string's buffer.
;;    You can set this to -1 (FFFFFFFFh) if string is quaranteed to be properly zero-terminated.
;; @arg allowplus
;;    If 1, explicit "+" sign before number is allowed.
;; @arg len
;;    Pointer to dword variable, which will on return hold number of converted characters.
;;    Ignored if zero.
;; @ret
;;    CF=1 on error, otherwise
;;    EAX = value.
;; @err ERR_INVALID_MODE
;;    allowplus > 1
;; @err ERR_NOT_DECIMAL
;;    First character of string is not decimal digit.
;; @err ERR_OUT_OF_RANGE
;;    Value is greater than 2147483647 or lesser than -2147483648.
;; @err ERR_ZERO_SIZE
;;    strlen=0
;; @note This procedure converts digits until it reaches non-digit character,
;;       or until value overflows.
;; @nodeps
proc extconv.dec_id string, buflen, allowplus, len
	libcall extconv.dec2id_size, [string], [buflen], 2147483648, [allowplus], [len]
	ret
endp