;; @name extconv.w_hex
;; @desc
;;    Converts unsigned word (16bit) value to hex string.
;;    Supports padding of output with spaces or zeroes.
;;    For simpler version see {conv.w_hex}.
;; @arg dest
;;    Pointer to destination string buffer.
;; @arg destlen
;;    Size of destination string buffer.
;; @arg value
;;    Word value to convert.
;;    Only low order word of this dword is used, rest is ignored.
;; @arg padding
;;    Type of padding:
;;    type 0 is no padding,
;;    type 1 is prepending with spaces,
;;    type 2 is prepending with leading zeroes.
;; @arg padlen
;;    Length to which should output be padded.
;;    If output is already longer than this value before padding, then output is not padded.
;;    Ignored if padding = 0.
;; @arg len
;;    Pointer to dword variable, which will on return hold length of output (terminating zero not included).
;;    Ignored if zero.
;; @ret
;;    CF=1 on error.
;; @err ERR_BUFFER_FULL
;;    Destination buffer is not big enough.
;;    Only first destlen bytes of resulting string were written.
;;    String in destination buffer is NOT zero terminated.
;;    Variable pointed by len contains destlen
;; @err ERR_ZERO_SIZE
;;    destlen=0.
;;    Variable pointed by len is set to 0.
;; @err ERR_INVALID_MODE
;;    padding > 2.
;;    Variable pointed by len is set to 0.
;; @nodeps
proc extconv.w_hex dest, destlen, value, padding, padlen, len
	movzx	eax, word [value]
	libcall extconv.d_hex, [dest], [destlen], eax, [padding], [padlen], [len]
	ret
endp