;macro to define error table
macro errtable [name, string]
{
  common
    local ..count
    ..count = 0
    err.table:
  forward
    local ..string
    if (used name) | INCLUDE_ALL_ERRORS eq 1
      if (used err.text) | INCLUDE_ALL_ERRORS eq 1
	dd ..string
      end if
      name = ..count
      ..count = ..count + 1
    end if
  forward
    idata \{
      if (used err.text & used name) | INCLUDE_ALL_ERRORS eq 1
	..string db string,0
      end if
    \}
  common
    err.count = ..count
}
include 'table.inc'


idata {
  if used err._invalid_error_code | INCLUDE_ALL_ERRORS eq 1
    err._invalid_error_code db "Internal Error: Invalid error code passed to err.text",0
  end if
}

;---------------------------------------------------------------------
;; @name err.text
;; @desc
;;    This procedure returns error message associated with error code.
;; @arg errcode
;;    This can be either {error codes:FASMLIB error code} or {custom error code}.
;; @ret
;;    CF=0 always, and
;;    EAX = pointer to zero-terminated ASCII string holding error message
proc err.text errcode

	mov	eax, [errcode]

	;error codes below err.count are defined in table
	cmp	eax, err.count
	ja	@f
	mov	eax, [err.table + 4*eax]
	jmp	.done
@@:

	;other error codes below 1000h are invalid
	cmp	eax, 1000h
	jae	@f
	mov	eax, err._invalid_error_code
	jmp	.done
	@@:

	;external error codes are pointers to error message
.done:
	clc
	ret
endp