;; @name fstream._peek
;; @desc
;;    This is {stream peek method:peek method} of {fstream:file stream}.
;;    Don't call this procedure directly, use {file.peek} instead.
;; @arg stream
;;    Handle of file stream to write to.
;; @arg count
;;    Number of bytes to peek.
;; @arg readlen
;;    Pointer to variable, which will hold number of chars in returned buffer.
;;    Only needed when function returns OF, otherwise it's equal to count.
;;    You can set this to 0, when you don't want handle case with OF=1.
;; @ret
;;    CF set on error, otherwise
;;    OF set if EOF was reached (less than count bytes readen), and
;;    EAX = pointer to buffer (READ ONLY, do not write there)
;; @err ERR_HANDLE_CORRUPT
;;    Stream is corrupted.
;;    This can happen if you close file handle used by stream.
;;    This can also happen if you overwrite stream object.
;; @err ERR_HANDLE_NOT_READABLE
;;    Stream is not readable.
;; @err ERR_UNKNOWN
;;    Platform-specific error during reading file.
;; @nodep stream._def_peek
proc fstream._peek stream, count, readlen

	;check if mode is 0 or 2
	mov	eax, [stream]
	mov	eax, [eax + FSTREAM.mode]
	cmp	eax, 1
	je	.error_stream_not_readable

	;call default read method
	libcall stream._def_peek, [stream], [count], [readlen], fstream._readbuf
	jc	.error_peek

.r:	ret
.rc:	stc
	jmp	.r

.error_stream_not_readable:
	mov	eax, ERR_HANDLE_NOT_READABLE
	jmp	.rc

.error_peek:

	;translate ERR_INVALID_HANDLE to ERR_HANDLE_CORRUPT.
	;if file handle in stream is invalid, them stream object is corrupted
	cmp	eax, ERR_INVALID_HANDLE
	jne	@f
	mov	eax, ERR_HANDLE_CORRUPT
	jmp	.rc
	@@:

	jmp	.rc

endp