;; @name fstream._write
;; @desc
;;    This is {stream write method:write method} of {fstream:file stream}.
;;    Don't call this procedure directly, use {file.write} instead.
;; @arg stream
;;    Handle of file stream to write to.
;; @arg buffer
;;    Pointer to data which we want to write.
;; @arg count
;;    Number of bytes to write.
;; @ret
;;    CF set on error.
;; @err ERR_HANDLE_CORRUPT
;;    Stream is corrupted.
;;    This can happen if you close file handle used by stream.
;;    This can also happen if you overwrite stream object.
;; @err ERR_HANDLE_NOT_WRITEABLE
;;    Stream is not readable.
;; @err ERR_UNKNOWN
;;    Platform-specific error during writing to file.
proc fstream._write, stream, buffer, count
	push	ebx

	;check if mode is 1 or 2
	mov	ebx, [stream]
	mov	eax, [ebx + FSTREAM.mode]
	test	eax, eax
	je	.error_stream_not_writeable

	;file pointer is ahead by number of bytes in read buffer
	;seek back to original position
	mov	eax, [ebx + FSTREAM.rbuf.cnt]
	test	eax, eax
	jz	@f
	neg	eax
	libcall fileio.seek, [ebx + FSTREAM.handle], eax, -1, 1
	jc	.error_seek
	@@:

	;invalidate read buffer
	mov	[ebx + FSTREAM.rbuf.cnt], 0
	mov	[ebx + FSTREAM.rbuf.pos], 0
	mov	[ebx + FSTREAM.rbuf.eof], 0

	;write data to file
	libcall fileio.write, [ebx + FSTREAM.handle], [buffer], [count]
	jc	.error_write
	ret

.r:	pop	ebx
	ret
.rc:	stc
	jmp	.r

.error_stream_not_writeable:
	mov	eax, ERR_HANDLE_NOT_WRITEABLE
	jmp	.rc

.error_seek:
.error_write:

	;translate ERR_INVALID_HANDLE to ERR_HANDLE_CORRUPT.
	;if file handle in stream is invalid, them stream object is corrupted
	cmp	eax, ERR_INVALID_HANDLE
	jne	@f
	mov	eax, ERR_HANDLE_CORRUPT
	jmp	.rc
	@@:

	jmp	.rc

endp
