;; @name file.open
;; @desc
;;    Opens existing file as stream.
;; @arg name
;;    Pointer to name of file to open. Must be zero-terminated string.
;; @arg mode
;;    Access mode to file: 0=read only, 1=write only, 2=read/write.
;; @ret
;;    CF = 1 on error, otherwise
;;    EAX = stream handle
;; @err ERR_FILE_NOT_FOUND
;;    File with specified name wasn't found.
;; @err ERR_CANT_OPEN_FILE
;;    File exists, but couldn't be opened.
;; @err ERR_INVALID_MODE
;;    mode > 2.
;; @err ERR_UNKNOWN
;;    Platform-specific error during opening file.
;; @warn Don't forget to close returned file stream with {stream.close}!
;; @warn Do not use original handle while it is used by stream.
;; @note This is simple version, see {file.extopen} for more options.
;; @note Opens file as stream, if you want direct file I/O use {fileio.open}.
proc file.open name, mode
	;check if mode is valid
	cmp	[mode], 2
	ja	.error_invalid_mode

	;we need read buffer if mode is 0 or 2
.check_read_buffer:
	cmp	[mode], 1
	je	.no_read_buffer
	push	dword FSTREAM_READ_BUFFER_SIZE
	push	dword FSTREAM_READ_BUFFER
	jmp	.read_buffer_done
.no_read_buffer:
	push	dword 0
	push	dword 0
.read_buffer_done:

	;we need write buffer if mode is 1 or 2
.check_write_buffer:
	cmp	[mode], 0
	je	.no_write_buffer
	push	dword FSTREAM_WRITE_BUFFER_SIZE
	push	dword FSTREAM_WRITE_BUFFER
	jmp	.write_buffer_done
.no_write_buffer:
	push	dword 0
	push	dword 0
.write_buffer_done:

	;call extended version
	libcall file.extopen, 0, [name], [mode]
	jc	.rc

.rnc:	clc
.r:	ret
.rc:	stc
	jmp	.r

.error_invalid_mode:
	mov	eax, ERR_INVALID_MODE
	jmp	.rc
endp