;; @name file.size
;; @desc
;;    Returns size of opened file stream.
;;    Returned value is 64bit value.
;; @arg handle
;;    File stream handle.
;; @ret
;;    CF set on error, otherwise
;;    EDX:EAX = size of file in bytes
;; @err ERR_INVALID_HANDLE
;;    Not a file stream.
;; @note This is platform-dependent procedure
proc file.size handle
local pos:QWORD
local size:QWORD

	push	ebx

	;EBX = file stream object
	mov	ebx, [handle]

	;check if this really is file stream
	cmp	[ebx + STREAM.vtab], fstream._vtab
	jne	.error_invalid_stream

	;save current position in file
	libcall file.tell, ebx
	jc	.rc
	mov	dword [pos], eax
	mov	dword [pos+4], edx

	;seek end of file
	libcall file.seek, ebx, 0, 0, 2
	jc	.rc

	;get position of end of file
	libcall file.tell, ebx
	jc	.rc
	mov	dword [size], eax
	mov	dword [size+4], edx

	;restore original position
	libcall file.seek, ebx, dword [pos], dword [pos+4], 0
	jc	.rc

	;return position of end of file
	mov	eax, dword [size]
	mov	edx, dword [size+4]

.rnc:	clc
.r:	pop	ebx
	ret
.rc:	stc
	jmp	.r

.error_invalid_stream:
	mov	eax, ERR_INVALID_HANDLE
	jmp	.rc
endp