;===================================================================
; internal - for testing heap
proc mem.test_heap
	push	ebx edx

	;is module initialized?
	cmp	[mem.initialized], 0
	je	.error_not_initialized

	;was heap corruption detected?
	cmp	[mem.heap_corrupt], 0
	jne	.error_heap_corrupt

	mov	ebx, [mem.heap.start]
	mov	edx, 0

.process_block:

	;check if block lies in heap
	mov	eax, ebx
	sub	eax, [mem.heap.start]
	jb	.error_heap_corrupt
	cmp	eax, [mem.heap.size]
	jae	.error_heap_corrupt

	;check block
	cmp	[ebx + HEAPBLOCK.sign], 'HB'
	jne	.error_heap_corrupt
	cmp	[ebx + HEAPBLOCK.used], 1
	ja	.error_heap_corrupt
	cmp	[ebx + HEAPBLOCK.prev], edx
	jne	.error_heap_corrupt

	;if used, check 0 < asize <= size
	cmp	[ebx + HEAPBLOCK.used], 1
	ja	.error_heap_corrupt
	jb	@f
	mov	eax, [ebx + HEAPBLOCK.asize]
	test	eax, eax
	je	.error_heap_corrupt
	cmp	eax, [ebx + HEAPBLOCK.size]
	ja	.error_heap_corrupt
	@@:

	;check if last block fills entire heap
	cmp	[ebx + HEAPBLOCK.last], 1
	ja	.error_heap_corrupt
	jb	@f
	lea	eax, [ebx + sizeof.HEAPBLOCK]
	add	eax, [ebx + HEAPBLOCK.size]
	sub	eax, [mem.heap.start]
	cmp	eax, [mem.heap.size]
	jne	.error_heap_corrupt
	@@:

	;if this was last block, end
	cmp	[ebx + HEAPBLOCK.last], 1
	je	.done

	;move to next block
	mov	edx, ebx
	add	ebx, [ebx + HEAPBLOCK.size]
	add	ebx, sizeof.HEAPBLOCK
	jmp	.process_block

.done:
	;does last block fill entire heap?
	lea	eax, [ebx + sizeof.HEAPBLOCK]
	add	eax, [ebx + HEAPBLOCK.size]
	sub	eax, [mem.heap.start]
	cmp	eax, [mem.heap.size]
	jne	.error_heap_corrupt

.rnc:	clc
.r:	pop	edx ebx
	ret
.rc:	stc
	jmp	.r

.error_not_initialized:
	mov	eax, ERR_MODULE_NOT_INITIALIZED
	jmp	.rc

.error_heap_corrupt:
	mov	[mem.heap_corrupt], 1
	mov	eax, ERR_HEAP_CORRUPT
	jmp	.rc
endp