;============================================================================
; mem.uninit
; desc: uninitalizes mem module (and it's heap manager)
; args: none
; ret: CF set on error
; error: ERR_UNKNOWN     - platform-specific error
;        ERR_MEMORY_LEAK - you didn't free some allocated block
;        +mem.free_heap (in platform-dependant part of mem module)
;============================================================================
proc mem.uninit

	;is module initialized?
	cmp	[mem.initialized], 0
	jz	.error_not_initialized

	cmp	[mem.heap_corrupt],0
	jne	.dont_test_heap

	; check heap
	libcall mem.test_heap
	jc	.rc

.dont_test_heap:

	; call platform-dependant free heap
	libcall mem.free_heap, [mem.heap.start], [mem.heap.size]
	jc	.rc

	; uninitialized
	mov	[mem.initialized], 0

	; return error if there is some memory leak
	cmp	[mem.alloc_count], 0
	ja	.error_memory_leak

.rnc:	clc
.r:	ret
.rc:	mov	[mem.initialized], 0
	stc
	ret

.error_memory_leak:

	mov	eax, ERR_MEMORY_LEAK
	jmp	.rc

.error_not_initialized:
	mov	eax, ERR_MODULE_NOT_INITIALIZED
	jmp	.rc
endp