;; @name mem.copy
;; @desc
;;    Copies contents block of memory of given size to another place.
;;    Supports overlapped copy (like "memmove" in C).
;; @arg dest
;;    pointer to destination place, to where the block will be copied
;; @arg src
;;    pointer to source place, from where the block will be copied
;; @arg size
;;    number of bytes to copy
;; @ret
;;    CF = 1 on error, otherwise
;;    EAX = dest
;; @err ERR_INVALID_POINTER
;;    some of pointers point to inaccessible area
;; @note Supports overlapped copy, eg. case when src < dest < src+size.
;; @note Does nothing when size=0
;; @note Memory management doesn't have to be initalized for this to work
proc mem.copy, dest, src, size
	push	ecx esi edi
	pushf

	mov	edi, [dest]
	mov	esi, [src]
	mov	ecx, [size]

	;just return when copying 0 bytes
	cmp	ecx, 0
	je	.done

	;check if dest area is overlapping src (when src < dest < src+size)
	cmp	esi, edi
	je	.done ;if dest=src, we don't need to move
	ja	.not_overlapping
	lea	eax, [esi+ecx]
	cmp	edi, eax
	jae	.not_overlapping


	;if it's overlapping
.overlapping:
	;start moving from end of buffers
	add	esi, ecx
	add	edi, ecx

	;classical unoptimized unalgined move
	std
	dec	esi
	dec	edi
	shr	ecx, 1
	jnc	@f
	movsb
@@:

	dec	esi
	dec	edi
	shr	ecx, 1
	jnc	@f
	movsw
@@:	jecxz	@f

	sub	esi, 2
	sub	edi, 2
	rep	movsd
@@:

	jmp	.done


.not_overlapping:

	;classical unoptimized unalgined move
	cld
	shr	ecx, 1
	jnc	@f
	movsb
@@:	shr	ecx, 1
	jnc	@f
	movsw
@@:	jecxz	@f
	rep	movsd
@@:
	;everything is okay
.done:	and	byte [esp],not 1     ;clear CF in stack
	mov	eax,[dest]

.r:	popf
	pop	edi esi ecx
	ret
endp