;=============================================================================
;; @name mem.findb
;; @desc
;;    Finds first occurence of given byte in block of memory.
;; @arg block
;;    Pointer to block in which we will search.
;; @arg blocksize
;;    Size of block in which we search.
;; @arg value
;;    Byte for which we will search.
;;    Only low order byte of this dword is used, rest is ignored.
;; @ret
;;    CF = 1 on error, otherwise
;;    ZF = 1 if not found, otherwise
;;    EAX = offset (not pointer) in block, where the byte was found.
;; @err ERR_INVALID_POINTER
;;    some of pointers point to inaccessible area
;; @err ERR_ZERO_SIZE
;;    blocksize=0
;; @note Returned EAX can be in range 0 to block+blocksize-1
;; @note Memory management doesn't have to be initalized for this to work
proc mem.findb block, blocksize, value
	push	ecx edi
	pushf

	;if blocksize=0, nothing found
	cmp	[blocksize], 0
	je	.not_found

	;search
	mov	ecx, [blocksize]
	mov	edi, [block]
	mov	al, byte [value]
	cld
	repne	scasb
	jne	.not_found

	;found
	mov	eax, [blocksize]
	sub	eax, ecx
	dec	eax
	jmp	.rnz

	;not found
.not_found:
	mov	eax, -1 ;for case someone forgets to check ZF
	jmp	.rz

.rnz:	and	byte [esp], not (1 shl 6) ;clear ZF
.rnc:	and	byte [esp], not 1 ;clear CF
.r:	popf
	pop	edi ecx
	ret
.rc:	or	byte [esp], 1 ;set CF
	jmp	.r
.rz:	or	byte [esp], 1 shl 6 ;set ZF
	jmp	.rnc
endp