;; @name mstream.extcreate
;; @desc
;;    Creates new {mstream:memory stream}.
;;    This is extended version. For simpler usage see {mstream.create}
;; @arg objbuf
;;    Pointer to buffer for {mstream:MSTREAM} object.
;;    If 0, object is allocated.
;; @arg buffer
;;    Pointer to buffer where memory stream begins
;; @arg bufsz
;;    Size of buffer of memory stream.
;; @ret
;;    CF set on error, otherwise
;;    EAX = stream handle
;; @warn
;;    Don't forget to close stream with {stream.close}.
;; @warn
;;    Don't access buffer directly while it is used by stream.
;; @note
;;    If you try to write data behind end of buffer, write method
;;    returns ERR_BUFFER_FULL.
proc mstream.extcreate objbuf, buffer, bufsz
	push	edi

	;EDI = stream object
	libcall stream.create,\
		[objbuf],\			      ;pass info from caller
		sizeof.MSTREAM - sizeof.STREAM,\      ;size of additional data
		mstream._vtab,\ 		      ;virtual table
		0,0,0,0 			      ;no buffering needed with memory stream
	jc	.rc
	mov	edi, eax

	;set data
	mov	eax, [buffer]
	mov	[edi + MSTREAM.buf], eax
	mov	eax, [bufsz]
	mov	[edi + MSTREAM.bufsz], eax
	mov	dword [edi + MSTREAM.pos], 0
	mov	dword [edi + MSTREAM.pos+4], 0

	mov	eax, edi
.rnc:	clc
.r:	pop	edi
	ret
.rc:	stc
	jmp	.r
endp