;; @name str.cat
;; @desc
;;    Concatenates (appends) string to end of another string.
;; @arg dest
;;    Pointer to destination string.
;; @arg destlen
;;    Size of destination string buffer.
;; @arg src
;;    Pointer to source string.
;; @arg srclen
;;    Size of source string buffer.
;;    You can set this to -1 (FFFFFFFFh) if string is quaranteed to be properly zero-terminated.
;; @ret
;;    CF set on error, otherwise
;;    EAX = dest.
;; @err ERR_BUFFER_OVERFLOW
;;    Destination buffer is not big enough.
;;    Only first destlen bytes of resulting string were written.
;;    String in destination buffer is NOT zero terminated.
;; @err ERR_ZERO_SIZE
;;    destlen=0 or srclen=0.
proc str.cat dest, destlen, src, srclen
	push	ecx

	;error if destlen=0
	cmp	[destlen], 0
	je	.error_destlen_zero

	;error if srclen=0
	cmp	[srclen], 0
	je	.error_srclen_zero

	;get length of dest string
	libcall str.len, [dest], [destlen]
	jc	.rc

	;if dest string doesn't contain ending zero, return with OF=1
	jo	.error_dest_buffer_full

	;ECX = remaining size of dest buffer
	mov	ecx, [destlen]
	sub	ecx, eax

	;EAX = place where to start appending
	add	eax, [dest]

	;copy
	libcall str.copy, eax, ecx, [src], [srclen]
	jc	.rc

	mov	eax, [dest]

.rnc:	clc
.r:	pop	ecx
	ret
.rc:	stc
	jmp	.r


.error_dest_buffer_full:
	mov	eax, ERR_BUFFER_FULL
	jmp	.rc

.error_destlen_zero:
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc

.error_srclen_zero:
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc

endp