;; @name str.catchar
;; @desc
;;    Concatenates (appends) character to end of string.
;; @arg string
;;    Pointer to destination string.
;; @arg buflen
;;    Size of destination string buffer.
;; @arg char
;;    Character which we are appending to string.
;;    Only low order byte of this dword is used, rest is ignored.
;; @ret
;;    CF set on error, otherwise
;;    EAX = string.
;; @err ERR_BUFFER_OVERFLOW
;;    Destination buffer is not big enough.
;;    Only first destlen bytes of string were written.
;;    String in destination buffer is NOT zero terminated.
;; @err ERR_ZERO_VALUE
;;    char=0
;; @err ERR_ZERO_SIZE
;;    strlen=0
proc str.catchar string, buflen, char
	push	edi ecx

	;error if buflen=0
	cmp	[buflen], 0
	je	.error_buflen_zero

	;error if char=0
	cmp	[char], 0
	je	.error_char_zero

	;EDI = string
	mov	edi, [string]

	;get length of string
	libcall str.len, [string], [buflen]
	jc	.rc
	jo	.error_dest_buffer_full

	;append char
	mov	cl, byte [char]
	mov	byte [edi+eax], cl

	;error if there is no more space for ending zero
	inc	eax
	cmp	eax, [buflen]
	je	.error_dest_buffer_full

	;append ending zero
	mov	byte [edi+eax], 0

	;return string
	mov	eax, edi
	clc

.r:	pop	ecx edi
	ret
.rc:	stc
	jmp	.r

.error_dest_buffer_full:
	mov	eax, ERR_BUFFER_FULL
	jmp	.rc

.error_buflen_zero:
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc

.error_char_zero:
	mov	eax, ERR_ZERO_VALUE
	jmp	.rc
endp