;; @name str.copy
;; @desc
;;    Copies string into string buffer.
;; @arg dest
;;    Pointer to destination string buffer.
;; @arg destlen
;;    Size of destination string buffer.
;; @arg src
;;    Pointer to source string.
;; @arg srclen
;;    Size of source string buffer.
;;    You can set this to -1 (FFFFFFFFh) if string is quaranteed to be properly zero-terminated.
;; @ret
;;    CF set on error, otherwise
;;    EAX = dest.
;; @err ERR_BUFFER_OVERFLOW
;;    Destination buffer is not big enough.
;;    Only first destlen bytes of string were written.
;;    String in destination buffer is NOT zero terminated.
;; @err ERR_ZERO_SIZE
;;    destlen=0 or srclen=0.
proc str.copy dest, destlen, src, srclen
	push	edi esi ecx ebx
	cld

	;EBX = destlen, error if destlen=0
	mov	ebx, [destlen]
	cmp	ebx, 0
	je	.error_destlen_zero

	;ECX = srclen, error if srclen=0
	mov	ecx, [srclen]
	cmp	ecx, 0
	je	.error_srclen_zero

	;copy at most max(EBX,ECX) characters, until ending zero
	mov	esi, [src]
	mov	edi, [dest]
.copy_char:
	lodsb
	stosb
	cmp	al, 0
	je	.done
	dec	ebx		;this test must be before src overflow test, so we can always
	jz	.dest_overflow	;add ending zero in .source_overflow
	dec	ecx
	jnz	.copy_char

	;end of source string buffer reached, without finding ending zero
.source_overflow:
	mov	byte [edi], 0		;add ending zero to dest string
	jmp	.done

	;end of destination buffer reached, before storing ending zero
.dest_overflow:
	mov	eax, ERR_BUFFER_FULL
	stc
	jmp	.r

	;done, eax=dest
.done:	mov	eax, [dest]
	clc

.r:	pop	ebx ecx esi edi
	ret

.error_destlen_zero:
	mov	eax, ERR_ZERO_SIZE
	stc
	jmp	.r

.error_srclen_zero:
	mov	eax, ERR_ZERO_SIZE
	stc
	jmp	.r
endp