;; @name str.len
;; @desc
;;    Returns length of string.
;;    If string isn't zero-terminated within it's buffer, size of buffer is returned,
;;    and OF is set.
;; @arg string
;;    Pointer to string.
;; @arg buflen
;;    Size of string buffer.
;;    You can set this to -1 (FFFFFFFFh) if string is quaranteed to be properly zero-terminated.
;; @ret
;;    CF set on error, otherwise
;;    EAX = length of string, and
;;    OF set if string isn't zero-terminated.
;; @err ERR_ZERO_SIZE
;;    buflen=0
proc str.len string, buflen
	push	ecx edi
	pushf
	cld

	;ERR_ZERO_SIZE if buflen=0
	cmp	[buflen], 0
	je	.error_buflen_zero

	;clear OF in stack
	and	word [esp], not (1 shl 11)

	;find ending zero in buffer
	mov	ecx, [buflen]
	mov	edi, [string]
	xor	al,al
	repne	scasb

	;set OF if ending 0 wasn't found
	je	@f
	or	word [esp], 1 shl 11
	dec	ecx		;to disable effect of "dec eax", if there was no ending zero (ugly, i know...)
@@:

	;EAX = number of readen chars
	mov	eax, [buflen]
	sub	eax, ecx
	dec	eax		;ending zero

	;clear CF
	and	byte [esp], not 1

.r:	popf
	pop	edi ecx
	ret

.error_buflen_zero:
	mov	eax, ERR_ZERO_SIZE
.rc:	or	byte [esp], 1	;set CF in stack
	jmp	.r

endp