;; @name str.pos
;; @desc
;;    Finds position of first occurence of string pattern inside another string.
;; @arg string
;;    Pointer to string in which we are searching.
;; @arg buflen
;;    Size of buffer of string in which we are searching.
;;    You can set this to -1 (FFFFFFFFh) if string is quaranteed to be properly zero-terminated.
;; @arg pattern
;;    Pointer to pattern string.
;; @arg patlen
;;    Size of pattern string buffer.
;;    You can set this to -1 (FFFFFFFFh) if string is quaranteed to be properly zero-terminated.
;; @ret
;;    CF set on error, otherwise
;;    ZF set if pattern wasn't found, otherwise
;;    EAX = index of first occurence of pattern within string.
;; @err ERR_ZERO_VALUE
;;    Pattern is empty string.
;; @err ERR_ZERO_SIZE
;;    buflen=0 or patlen=0.
proc str.pos	string, buflen, pattern, patlen
	push	ebx
	pushf
	cld

	;error if buflen=0
	cmp	[buflen], 0
	je	.error_buflen_zero

	;error if patlen=0
	cmp	[patlen], 0
	je	.error_patlen_zero

	;ebx = length of the string to search
	libcall str.len, [string], [buflen]
	jc	.rc
	mov	ebx,eax

	;not found if ebx=0
	cmp	ebx, 0
	je	.not_found

	;edx = length of the pattern string
	libcall str.len, [pattern], [patlen]
	jc	.rc

	;error if it is empty
	cmp	eax, 0
	je	.error_pattern_empty

	;use mem.find to find string
	libcall mem.find, [string], ebx, [pattern], eax
	jc	.rc
	jz	.not_found


.found:
	and	dword [esp],not 1	  ;clear CF
	and	dword [esp],not (1 shl 6) ;clear ZF
	jmp	.r

.not_found:
	and	dword [esp],not 1	  ;clear CF
	or	dword [esp],1 shl 6	  ;set ZF

.r:
	popf
	pop	ebx
	ret

.rc:
	or	dword [esp],1		  ;set CF
	jmp	.r

.error_buflen_zero:
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc

.error_patlen_zero:
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc

.error_pattern_empty:
	mov	eax, ERR_ZERO_VALUE
	jmp	.rc
endp