;; @name stream.tell
;; @desc
;;    Returns current position in stream.
;;    Position is 64bit value.
;; @arg stream
;;    Handle of stream.
;; @ret
;;    CF set on error, otherwise
;;    EDX:EAX = current position in file.
;; @err ERR_HANDLE_NOT_SEEKABLE
;;    This type of stream doesn't support seeking.
;; @dep {stream tell method}
proc stream.tell stream

	;EAX = stream virtual table
	mov	eax, [stream]
	mov	eax, [eax + STREAM.vtab]

	;check if this is proper stream
	cmp	[eax + STREAM_VTAB.sign], 'STRM'
	jne	.error_not_stream

	;EAX = tell method, error if not present
	mov	eax, [eax + STREAM_VTAB.tell]
	cmp	eax, 0
	je	.error_stream_not_seekable

	;call method
	libcall eax, [stream]
	jc	.rc

.rnc:	clc
.r:	ret
.rc:	stc
	jmp	.r

.error_not_stream:
	mov	eax, ERR_INVALID_HANDLE
	jmp	.rc

.error_stream_not_seekable:
	mov	eax, ERR_HANDLE_NOT_SEEKABLE
	jmp	.rc
endp