;; @name stream.write
;; @desc
;;    Writes data from buffer to stream.
;; @arg stream
;;    Handle of stream to which we write.
;; @arg buffer
;;    Pointer to data to write.
;; @arg count
;;    Number of bytes to write.
;;    Procedure does nothing if count=0.
;; @ret
;;    CF set on error
;; @err ERR_INVALID_HANDLE
;;    Not a valid stream handle.
;; @err ERR_STREAM_NOT_WRITABLE
;;    Stream is not writeable.
;; @dep {stream write method}
proc stream.write, stream, buffer, count

	;do nothing if count is zero
	cmp	[count], 0
	je	.rnc

	;EAX = stream virtual table
	mov	eax, [stream]
	mov	eax, [eax + STREAM.vtab]

	;check if this is proper stream
	cmp	[eax + STREAM_VTAB.sign], 'STRM'
	jne	.error_not_stream

	;EAX = write method, error if not present
	mov	eax, [eax + STREAM_VTAB.write]
	cmp	eax, 0
	je	.error_stream_not_writeable

	;call write method
	libcall eax, [stream], [buffer], [count]
	jc	.rc

.rnc:	clc
.r:	ret
.rc:	stc
	jmp	.r

.error_not_stream:
	mov	eax, ERR_INVALID_HANDLE
	jmp	.rc

.error_stream_not_writeable:
	mov	eax, ERR_HANDLE_NOT_WRITEABLE
	jmp	.rc
endp