;=============================================================================
; text.extread.dec2d_size
; desc: INTERNAL! does job dec2b, dec2w, dec2d
;=============================================================================
proc text.extread.dec2d_size stream, size, maxlen, skipbl, len
local buflen dd ?
	push	ebx ecx edx esi

	;skip blank chars
	;smartass thingy, but without it it would confuse people
	cmp	[skipbl], 1
	ja	.error_invalid_mode
	jne	@f
	libcall text.skip.chars, [stream], text.blank_chars
	jc	.rc
	@@:

	;EDX = maximum number of chars allowed to read  (0 if not specified)
	mov	edx, -1
	mov	eax, [maxlen]
	cmp	eax, 0
	cmovne	edx, eax

	;peek at first char
	libcall stream.peek.byte, [stream]
	jc	.rc
	jo	.error_eof

	;error if it isn't dec char
	cmp	al, '0'
	jb	.error_not_numeric
	cmp	al, '9'
	ja	.error_not_numeric

	;ECX = number of char converted
	xor	ecx, ecx

	;EBX = result
	xor	ebx, ebx

.load_char:

	;peek 1 more char
	lea	eax, [ecx+1]
	libcall stream.peek, [stream], eax, addr buflen
	jc	.rc
	jo	.done
	mov	esi, eax

.convert_char:
	movzx	eax, byte [esi + ecx]

	;end if it isn't hex digit
	cmp	al, '0'
	jb	.done
	cmp	al, '9'
	ja	.done

	;convert to hex digit
	sub	al, '0'

	;miltply value by 10, and add converted digit to it
	push	edx eax esi
	xor	edx, edx
	mov	esi, 10
	mov	eax, ebx
	mul	esi
	pop	esi
	mov	ebx, eax
	pop	eax
	add	ebx, eax
	mov	eax, edx
	pop	edx

	;check for overflow
	jc	.error_overflow
	cmp	eax, 0
	ja	.error_overflow
	cmp	ebx, [size]
	ja	.error_overflow

	;check if we have read enough chars
	inc	ecx
	cmp	ecx, edx
	ja	.error_internal
	je	.done

	;if we don't have another char buffered, read it. otherwise continue converting next char
	cmp	ecx, [buflen]
	ja	.error_internal
	jb	.convert_char
	jmp	.load_char

.done:
	;skip readen chars
	libcall stream.skip, [stream], ecx
	jc	.rc
	jo	.error_internal

	;save number of chars readen to variable pointet by len, if desired
	mov	eax, [len]
	cmp	eax, 0
	je	@f
	mov	[eax], ecx
	@@:

	;store result
	mov	eax, ebx

	clc
.r:	pop	esi edx ecx ebx
	ret
.rc:	stc
	jmp	.r


.error_invalid_mode:
	;save 0 to variable pointed by len, if desired
	mov	eax, [len]
	cmp	eax, 0
	je	@f
	mov	dword [eax], 0
	@@:

	mov	eax, ERR_INVALID_MODE
	jmp	.rc

.error_eof:
	;save 0 to variable pointet by len, if desired
	mov	eax, [len]
	cmp	eax, 0
	je	@f
	mov	dword [eax], 0
	@@:

	mov	eax, ERR_HANDLE_EOF
	jmp	.rc

.error_not_numeric:
	;save 0 to variable pointet by len, if desired
	mov	eax, [len]
	cmp	eax, 0
	je	@f
	mov	dword [eax], 0
	@@:

	mov	eax, ERR_NOT_DECIMAL
	jmp	.rc

.error_internal:
	mov	eax, ERR_FASMLIB
	jmp	.rc

.error_len_zero:
	;save 0 to variable pointet by len
	mov	eax, [len]
	cmp	eax, 0
	je	@f
	mov	dword [eax], 0
	@@:

	mov	eax, ERR_ZERO_VALUE
	jmp	.rc

.error_overflow:
	;save ecx to variable pointet by len
	mov	eax, [len]
	cmp	eax, 0
	je	@f
	mov	[eax], ecx
	@@:

	mov	eax, ERR_OUT_OF_RANGE
	jmp	.rc
endp