;; @name text.extread.dec_iw
;; @desc
;;    Reads signed decimal word from stream.
;;    This is extended version, for simpler usage see {text.read.dec_iw}.
;;    Reads decimal digit characters ('0'-'9') until it reaches non-digit, EOF,
;;    or until resulting value overflows.
;; @arg stream
;;    Handle of stream from which to read.
;; @arg maxlen
;;    Maximum number of characters to read, including sign.
;;    0 if not specified.
;; @arg skipbl
;;    If 1, procedure first skips initial blank characters (space, tab, EOL).
;;    These characters don't count into maxlen size.
;;    These characters do not remain on stream in case of error.
;; @arg allowplus
;;    If 1, plus sign "+" is accepted.
;; @arg len
;;    Pointer to dword variable, that will be set to number of digits read.
;;    Ignored if 0.
;;    When skipbl=1, skipped initial blank characters don't count.
;; @ret
;;    CF set on error, otherwise
;;    EAX = readen word value, sign-extended to dword.
;; @err ERR_HANDLE_EOF
;;    EOF reached. len is set to 0.
;; @err ERR_OUT_OF_RANGE
;;    value < -32768 or value > 32767.
;;    In this case sign character and all digit characters remain on stream.
;;    len contains number of characters readen until overflow.
;; @err ERR_NOT_DECIMAL
;;    First character (after sign) is not a decimal digit.
;;    Characters remains on stream.
;;    len is set to 0.
;; @err ERR_ZERO_VALUE
;;    maxlen=0.
;;    len is set to 0.
;; @err ERR_INVALID_MODE
;;    skipbl > 1.
;;    len is set to 0.
;; @note Stream must have a read buffer.
;; @note If stream has static read buffer, it must be big enough to hold all characters
;;       of number.
;; @nodep text.extread.dec2id_size
proc text.extread.dec_iw stream, maxlen, skipbl, allowplus, len
	libcall text.extread.dec2id_size, [stream], 32768, [maxlen], [skipbl], [allowplus], [len]
	ret
endp