;; @name text.extskip.chars
;; @desc
;;     Skips all characters which are in given character list from stream.
;;     This is extended version of procedure, for simpler version see {text.skip.chars}.
;; @arg stream
;;     Stream to skip characters from.
;; @arg charlist
;;     Null-terminated string containing list of characters to read.
;; @arg maxlen
;;     Maximum number of characters to skip.
;;     Ignored if 0.
;; @ret
;;    CF=1 on error, otherwise
;;    OF=1 if we are on EOF, otherwise
;;    EAX = number of characters skipped
;; @note
;;    This function never skips character 0 from stream.
proc text.extskip.chars stream, charlist, maxlen
local buflen dd ?
	push	ecx edx esi
	pushf

	;EDX = maximum number of chars allowed to skip (0 if not specified)
	mov	edx, -1
	mov	eax, [maxlen]
	cmp	eax, 0
	cmovne	edx, eax

	;ECX = number of chars skipped
	xor	ecx, ecx

	;check if we are on EOF
	libcall stream.peek.byte, [stream]
	jc	.rc
	jo	.eof

.skip_char:

	;peek 1 char
	libcall stream.peek.byte, [stream]
	jc	.rc
	jo	.done

	;done if it is character 0
	test	al, al
	jz	.done

	;find out if current char is in charlist
	libcall str.poschar, [charlist], -1, eax
	jc	.rc

	;if no, skipping is done
	jz	.done

	;skip char
	libcall stream.skip, [stream], 1
	jc	.rc
	jo	.error_internal

	;end loop if we skipped enough characters
	inc	ecx
	cmp	ecx, edx
	je	.done
	ja	.error_internal

	;continue loop
	jmp	.skip_char

.eof:
	;return ECX, OF=1
	mov	eax, ecx
	jmp	.ro

.done:

	;return ECX, OF=0
	mov	eax, ecx

.rno:	and	word [esp], not (1 shl 11) ;clear OF
.rnc:	and	byte [esp], not 1 ;clear CF
.r:	popf
	pop	esi edx ecx
	ret
.rc:	or	byte [esp], 1 ;set CF
	jmp	.r
.ro:	or	word [esp], (1 shl 11) ;set OF
	jmp	.rnc

.error_internal:
	mov	eax, ERR_FASMLIB
	jmp	.rc

endp
