;; @name text.extwrite.ib_hex
;; @desc
;;    Write signed byte to stream as hexadecimal string.
;; @arg stream
;;    Handle of stream to write to.
;; @arg value
;;    Value to write as hexadecimal string.
;;    Only low order byte of this dword is used, rest is ignored.
;; @arg padding
;;    Type of padding:
;;    Type 0 is no padding.
;;    Type 1 is prepending with spaces, and sign counts into padding size.
;;    Type 2 is prepending with leading zeroes. In this type padlen specifies number
;;    digits displayed. One more character is always printed as sign (' ', '+' or '-').
;; @arg padlen
;;    Length to which should output be padded.
;;    If output is already longer than this value before padding, then output is not padded.
;;    Ignored if padding = 0.
;; @arg plus
;;    Set to 1 if you want to display plus sign on positive numbers.
;; @ret
;;    CF=1 on error.
;; @err ERR_OUT_OF_RANGE
;;    padlen > 4096
;; @err ERR_INVALID_MODE
;;    padding > 2, or plus > 1
;; @note This is extended version of procedure, for simpler usage see {text.write.ib_hex}.
;; @note For unsigned version of this procedure, see {text.extwrite.b_hex}.
proc text.extwrite.ib_hex stream, value, padding, padlen, plus
	movsx	eax, byte [value]
	libcall text.extwrite.id_hex, [stream], eax, [padding], [padlen], [plus]
	ret
endp