;; @name text.extwrite.id_dec
;; @desc
;;    Write signed dword to stream as decimal string.
;; @arg stream
;;    Handle of stream to write to.
;; @arg value
;;    Value to write as decimal string.
;; @arg padding
;;    Type of padding:
;;    Type 0 is no padding.
;;    Type 1 is prepending with spaces, and sign counts into padding size.
;;    Type 2 is prepending with leading zeroes. In this type padlen specifies number
;;    digits displayed. One more character is always printed as sign (' ', '+' or '-').
;; @arg padlen
;;    Length to which should output be padded.
;;    If output is already longer than this value before padding, then output is not padded.
;;    Ignored if padding = 0.
;; @arg plus
;;    Set to 1 if you want to display plus sign on positive numbers.
;; @ret
;;    CF=1 on error.
;; @err ERR_OUT_OF_RANGE
;;    padlen > 4096
;; @err ERR_INVALID_MODE
;;    padding > 2, or plus > 1
;; @note This is extended version of procedure, for simpler usage see {text.write.id_dec}.
;; @note For unsigned version of this procedure, see {text.extwrite.d_dec}.
proc text.extwrite.id_dec stream, value, padding, padlen, plus
local convlen dd ?

	push	ebx ecx

	;ECX = size of needed buffer, at least 16
	mov	ecx, [padlen]
	inc	ecx
	inc	ecx
	add	ecx, 3
	and	ecx, not 3  ;align to dword
	mov	eax, 16
	cmp	ecx, eax
	cmovb	ecx, eax

	;sanity-check for padlen > 4096, to prevent big allocation on stack
	cmp	ecx, 4096
	ja	.error_padlen_too_big

	;EBX = allocte padlen+1 buffer on stack
	sub	esp, ecx
	mov	ebx, esp

	;convert
	libcall extconv.id_dec, ebx, ecx, [value], [padding], [padlen], [plus], addr convlen
	jc	.error_convert

	;display
	libcall stream.write, [stream], ebx, [convlen]
	jc	.error_write

	;remove buffer from stack
	add	esp, ecx

.r:	pop	ebx
	ret
.rc:	stc
	jmp	.r

.error_write:
	add	esp, ecx
	jmp	.rc

.error_padlen_too_big:
	mov	eax, ERR_OUT_OF_RANGE
	jmp	.rc

.error_convert:
	add	esp, ecx
	;if returned error is ERR_BUFFER_FULL, it's FASMLIB bug
	cmp	eax, ERR_BUFFER_FULL
	jne	.rc
	mov	eax, ERR_FASMLIB
	jmp	.rc
endp