;; @name text.fetch.str
;; @desc
;;    Reads specified ASCII string off the stream.
;;    If there is another string on stream, then nothing is read and ZF=1 is returned.
;;    If string is guaranteed to be null terminated, you can use {text.fetch} instead.
;; @arg stream
;;    Handle of stream to read from.
;; @arg string
;;    String to fetch from stream.
;; @arg buflen
;;    Length of string buffer.
;; @ret
;;    CF set on error, otherwise
;;    ZF set if the string was fetched
;; @note For standard input, you can call {stdin.fetch.str} instead of passing {stdin} as argument.
proc text.fetch.str stream, string, buflen
local len dd ?

	push	ebx ecx edx esi edi

	;ESI = string
	mov	esi, [string]

	;find length of string (sorry, lazy and slow way)
	libcall str.len, esi, [buflen]
	jc	.rc
	mov	ecx, eax

	;EBX = index of character we are comparing
	xor	ebx, ebx

.peek_more:

	;blocking-peek one more character
	lea	eax, [ebx+1]
	libcall stream.peek, [stream], eax, addr len
	jc	.rc
	jo	.not_match
	mov	edx, [len]

	;TODO: try to peeknb rest

.compare:
	mov	edi, eax

.compare_char:
	;compare character
	mov	al, [edi+ebx]
	cmp	al, [esi+ebx]
	jne	.rnc   ;return ZF=0

	;are we finished
	inc	ebx
	cmp	ebx, ecx
	ja	.error_internal
	je	.fetched

	;continue comparing loop
	cmp	ebx, edx
	ja	.error_internal
	jb	.compare_char

	;we need to peek some more characters
	jmp	.peek_more

.fetched:

	;skip it from stream
	libcall stream.skip, [stream], ecx
	jc	.rc
	jo	.error_internal

	;return ZF
	cmp	eax, eax
	jmp	.rnc

.not_match:

	;clean ZF
	cmp	ecx, -1  ;string probably won't be this big :)

.rnc:	clc
.r:	pop	edi esi edx ecx ebx
	ret
.rc:	or	byte [esp], 1 ;set CF
	jmp	.r

.error_internal:
	mov	eax, ERR_FASMLIB
	jmp	.rc

endp
