;; @name text.read.dec_iq
;; @desc
;;    Reads signed decimal qword from stream.
;;    This is simple version, for more options see {text.extread.dec_iq}.
;;    Reads optional sign ('-' or '+') and decimal digit characters ('0'-'9') until
;;    it reaches non-digit, EOF, or until resulting value overflows.
;; @arg stream
;;    Handle of stream from which to read.
;; @ret
;;    CF set on error, otherwise
;;    EDX:EAX = readen qword
;; @err ERR_HANDLE_EOF
;;    EOF reached.
;; @err ERR_OUT_OF_RANGE
;;    value < -9223372036854775808 or value > 9223372036854775807.
;;    In this case sign character and all digit characters remain on stream.
;; @err ERR_NOT_DECIMAL
;;    First character (after sign) is not decimal digit.
;;    Character remains on stream.
;; @note Skips inital blank characters (space, tab, EOL).
;; @note Stream must have a read buffer.
;; @note If stream has static read buffer, it must be big enough to hold all digits
;;       of number.
proc text.read.dec_iq stream
	libcall text.extread.dec_iq, [stream], 0, 1, 1, 0
	ret
endp