;; @name text.read.hex_b
;; @desc
;;    Reads unsigned hexadecimal byte from stream.
;;    This is simple version, for more options see {text.extread.hex_b}.
;;    Reads hexadecimal digit characters ('0'-'9','a'-'z','A'-'Z') until it reaches non-digit, EOF,
;;    or until resulting value overflows.
;; @arg stream
;;    Handle of stream from which to read.
;; @ret
;;    CF set on error, otherwise
;;    EAX = readen byte zero-extended to dword
;; @err ERR_HANDLE_EOF
;;    EOF reached.
;; @err ERR_OUT_OF_RANGE
;;    value > FF.
;;    In this case all digit characters remain on stream.
;; @err ERR_NOT_HEXADECIMAL
;;    First character is not a hexadecimal digit.
;;    Character remains on stream.
;; @note Skips inital blank characters (space, tab, EOL).
;; @note Stream must have a read buffer.
;; @note If stream has static read buffer, it must be big enough to hold all digits
;;       of number.
;; @nodep text.extread.hex2d_size
proc text.read.hex_b stream
	push	0		;skip blank chars
	push	1		;no len
	push	0		;no maxlen
	push	0x000000F0	;overflow
	push	[stream]	;stream we read from
	call	text.extread.hex2d_size
	ret
endp
