;; @name text.read.line
;; @desc
;;     Reads one line of text from stream to buffer.
;;     This is simpler version of procedure, for extended version see {text.extread.line}.
;; @arg stream
;;     Stream to read characters from.
;; @arg buffer
;;     Pointer to destination buffer, where the characters will be stored.
;; @arg buflen
;;     Length of destination buffer.
;; @ret
;;    CF=1 on error, otherwise
;;    EAX = number of characters readen, and
;;    ZF=1 if EAX=0 (no characters readen).
;; @note Supports both LF and CRLF end of line combinations.
;; @note End of line character is not stored in resulting string.
;; @err ERR_ZERO_SIZE
;;    buflen=0.
;; @err ERR_BUFFER_FULL
;;    Destiantion buffer is full.
;;    Only buflen characters readen, and string in destination buffer is
;;    NOT zero-terminated.
;; @err ERR_HANDLE_EOF
;;    EOF reached without reading anything.
proc text.read.line stream, buffer, buflen
	libcall text.extread.line, [stream], [buffer], [buflen], 0, 0
	ret
endp